<?php

namespace App\Http\Controllers\Api\V1;

use App\Http\Controllers\Controller;
use App\Http\Requests\ClientNomineeRequest;
use App\Http\Resources\ClientNomineeResource;
use App\Models\ClientNominee;
use App\Models\Lead;
use Illuminate\Http\JsonResponse;

class ClientNomineeController extends Controller
{
    protected array $fileFieldNames = [
        "aadhar", "pan",
    ];

    protected array $fileFolderPaths = [
        "aadhar-cards", "pans",
    ];

    protected array $relations = [
        "client",
    ];

    /**
     * Display a listing of the clientNominees.
     */
    public function index(): JsonResponse {
        $clientNominees = $this->paginateOrGet(ClientNominee::with($this->relations)->latest());

        return $this->respondWithResourceCollection(
            ClientNomineeResource::collection($clientNominees)
        );
    }

    /**
     * Store a newly created clientNominee in storage.
     */
    public function store(ClientNomineeRequest $request): JsonResponse {
        // fetch the client from the request or database
        $client = Lead::find($request->client_id);

        // check if client already has 3 nominees
        if ($client->clientNominees()->count() >= Lead::MAX_NOMINEES) {
            return $this->respondError('Client already has the maximum of 3 nominees.', null, 422);
        }

        // check if the relation already exists for the client
        $existingNomineeWithSameRelation = $client->clientNominees()
            ->where('relation', $request->relation)
            ->exists();

        if ($existingNomineeWithSameRelation) {
            return $this->respondError('A nominee with this relation already exists for the client.', null, 422);
        }

        $clientNominee = new ClientNominee($request->validated());
        $this->storeFiles($request, $clientNominee);
        $clientNominee->save();

        return $this->respondCreated(
            new ClientNomineeResource($clientNominee),
            "Client's Nominee created successfully!"
        );
    }

    /**
     * Display the specified clientNominee.
     */
    public function show(ClientNominee $clientNominee): JsonResponse {
        $clientNominee->loadMissing($this->relations);

        return $this->respondWithResource(new ClientNomineeResource($clientNominee));
    }

    /**
     * Update the specified clientNominee in storage.
     */
    public function update(ClientNomineeRequest $request, ClientNominee $clientNominee): JsonResponse {
        // check if the relation already exists for the client
        $existingNomineeWithSameRelation = $clientNominee->client->clientNominees()
            ->where('relation', $request->relation)
            ->exists();

        if ($existingNomineeWithSameRelation) {
            return $this->respondError('A nominee with this relation already exists for the client.', null, 422);
        }

        $clientNominee->fill($request->validated());
        $this->updateFiles($request, $clientNominee);
        $clientNominee->save();

        return $this->respondUpdated(
            new ClientNomineeResource($clientNominee),
            "Client's Nominee updated successfully!"
        );
    }

    /**
     * Remove the specified clientNominee from storage.
     */
    public function destroy(ClientNominee $clientNominee): JsonResponse {
        $clientNominee->delete();

        return $this->respondSuccess("Client's Nominee deleted successfully!");
    }
}
