<?php

namespace App\Http\Controllers\Api\V1;

use App\Enums\LeadStatus;
use App\Enums\UserRole;
use App\Http\Controllers\Controller;
use App\Models\CommissionSlab;
use App\Models\DailyUpdate;
use App\Models\Lead;
use App\Models\User;
use Carbon\Carbon;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;

class DashboardController extends Controller
{
    protected array $months = [
        'January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October',
        'November', 'December'
    ];

    /**
     * Handle the incoming request.
     */
    public function __invoke(Request $request) {
        // get the currently logged-in user
        $user = Auth::user();

        if ($user->role === UserRole::CLIENT->value) {
            return $this->getClientDashboard($user);
        }

        if ($user->role === UserRole::DISTRIBUTOR->value) {
            return $this->getDistributorDashboard($user);
        }

        // Otherwise, return the regular dashboard data
        return $this->getAdminDashboard();
    }

    /**
     * Get data for Client Dashboard (last 30 days of DailyUpdates).
     */
    protected function getClientDashboard(User $user) {
        // fetch last 30 days of entries from DailyUpdates for this client
        $last30Days = DailyUpdate::where('client_id', $user->id)
            ->where('date', '>=', Carbon::now()->subDays(30)) // Entries within the last 30 days
            ->get();

        return $this->respondSuccess(
            null,
            $last30Days
        );

        // generate an array of the last 30 days
//        $last30Days = collect();
//        for ($i = 29; $i >= 0; $i--) {
//            $last30Days->push(Carbon::now()->subDays($i)->format('d/m/Y'));
//        }
//
//        // fetch the DailyUpdates for the last 30 days for this client
//        $dailyUpdates = DailyUpdate::where('client_id', $user->id)
//            ->where('date', '>=', Carbon::now()->subDays(30)) // Entries within the last 30 days
//            ->get()
//            ->mapWithKeys(function ($update) {
//                return [Carbon::parse($update->date)->format('d/m/Y') => $update->amount];
//            });
//
//        // map the results to include all dates and use 0 for dates without records
//        $results = $last30Days->map(function ($date) use ($dailyUpdates) {
//            return [
//                'date' => $date,
//                'value' => $dailyUpdates->get($date, 0) // use 0 if the date is not found in dailyUpdates
//            ];
//        });
//
//        return $this->respondSuccess(
//            null,
//            $results->toArray()
//        );
    }

    /**
     * Get data for Distributor Dashboard (last 30 days of DailyUpdates).
     */
    protected function getDistributorDashboard(User $user) {
        $totalLeads = $user->distributorLeads()
            ->count();

        $convertedLeads = $user->distributorLeads()
            ->where("lead_status", LeadStatus::OPPORTUNITY->value)
            ->count();

        $total_aum = $user->distributorLeads()
            ->where("is_client", true)
            ->whereHas("user", function (Builder $query) {
                $query->where("created_at", ">=", Carbon::now()->subDays(30));
            })->sum("current_invested_amount");

        $slab = CommissionSlab::where('min_amount', '<=', $total_aum)
            ->where(function ($query) use ($total_aum) {
                $query->whereNull('max_amount')
                    ->orWhere('max_amount', '>', $total_aum);
            })->first();

        $commissionDue = $slab ? ($total_aum * $slab->trail_commission) / 100 : 0;

        $result = [
            "commission_due" => $commissionDue,
            "total_leads" => $totalLeads,
            "converted_leads" => $convertedLeads,
        ];

        return $this->respondSuccess(
            null,
            $result
        );
    }

    /**
     * Get data for Admin or other roles' Dashboard.
     */
    protected function getAdminDashboard() {
        $currentYear = Carbon::now()->year;

        // Fetch monthly converted leads for the current year
        $convertedLeadsMonthly = Lead::where('is_client', true)
            ->whereYear('converted_at', $currentYear)
            ->selectRaw('MONTHNAME(converted_at) as month, COUNT(*) as count')
            ->groupBy('month')
            ->get()
            ->mapWithKeys(fn($converted) => [$converted->month => $converted->count]);
        $convertedMonthlyFormatted = $this->formatWithDefaults($convertedLeadsMonthly, $this->months);

        // Total leads for the current year
        $totalLeads = Lead::where('is_client', true)
            ->whereYear('converted_at', $currentYear)
            ->count();

        // Fetch monthly clients for the current year
        $clientsMonthly = User::where("role", UserRole::CLIENT->value)
            ->whereYear('created_at', $currentYear)
            ->selectRaw("MONTHNAME(created_at) as month, COUNT(*) as count")
            ->groupBy('month')
            ->get()
            ->mapWithKeys(fn($client) => [$client->month => $client->count]);
        $clientsMonthlyFormatted = $this->formatWithDefaults($clientsMonthly, $this->months);

        // Total clients for the current year
        $totalClients = User::where("role", UserRole::CLIENT->value)
            ->whereYear('created_at', $currentYear)
            ->count();

        // Fetch monthly distributors for the current year
        $distributorsMonthly = User::where("role", UserRole::DISTRIBUTOR->value)
            ->whereYear('created_at', $currentYear)
            ->selectRaw("MONTHNAME(created_at) as month, COUNT(*) as count")
            ->groupBy('month')
            ->get()
            ->mapWithKeys(fn($distributor) => [$distributor->month => $distributor->count]);
        $distributorsMonthlyFormatted = $this->formatWithDefaults($distributorsMonthly, $this->months);

        // Total distributors for the current year
        $totalDistributors = User::where("role", UserRole::DISTRIBUTOR->value)
            ->whereYear('created_at', $currentYear)
            ->count();

        // Return the response with monthly and total counts
        return $this->respondSuccess(
            null,
            [
                "leads" => $convertedMonthlyFormatted,
                "total_leads" => $totalLeads,
                "clients" => $clientsMonthlyFormatted,
                "total_clients" => $totalClients,
                "distributors" => $distributorsMonthlyFormatted,
                "total_distributors" => $totalDistributors,
            ]
        );
    }

    /**
     * Format data with default months and counts.
     */
    protected function formatWithDefaults($items, $defaults) {
        return collect($defaults)->map(fn($item) => [
            'month' => $item,
            'count' => $items->get($item, 0)
        ])->values();
    }
}
