<?php

namespace App\Http\Controllers\Api\V1;

use App\Enums\DistributorStatus;
use App\Enums\UserRole;
use App\Http\Controllers\Controller;
use App\Http\Requests\DistributorRequest;
use App\Http\Resources\DistributorResource;
use App\Mail\WelcomeMail;
use App\Models\Distributor;
use Illuminate\Http\JsonResponse;
use Illuminate\Support\Facades\Mail;

class DistributorController extends Controller
{
    protected array $relations = [
        "user", "approvedBy",
    ];

    /**
     * Display a listing of the distributors.
     */
    public function index(): JsonResponse {
        $distributors = $this->paginateOrGet(
            Distributor::with($this->relations)->latest());

        return $this->respondWithResourceCollection(
            DistributorResource::collection($distributors)
        );
    }

    /**
     * Store a newly created distributor in storage.
     */
    public function store(DistributorRequest $request): JsonResponse {
        $distributor = Distributor::create($request->validated());

        if ($distributor->status === DistributorStatus::APPROVED->value) {
            $this->createUserAndSendMail($distributor);
        }

        return $this->respondCreated(
            new DistributorResource($distributor),
            "Distributor created successfully!"
        );
    }

    private function createUserAndSendMail(Distributor $distributor) {
        $distributorUser = $distributor->user()->make($distributor->toArray());
        $distributorUser->password = $distributor->mobile_no;
        $distributorUser->role = UserRole::DISTRIBUTOR->value;
        $distributorUser->save();

        Mail::to($distributorUser->email)->send(new WelcomeMail($distributorUser));
        $distributor->is_mailed = true;
        $distributor->save();
    }

    /**
     * Display the specified distributor.
     */
    public function show(Distributor $distributor): JsonResponse {
        $distributor->loadMissing($this->relations);

        return $this->respondWithResource(new DistributorResource($distributor));
    }

    /**
     * Update the specified distributor in storage.
     */
    public function update(DistributorRequest $request, Distributor $distributor): JsonResponse {
        $distributor->update($request->validated());

        if ($distributor->status === DistributorStatus::APPROVED->value && !$distributor->is_mailed) {
            $this->createUserAndSendMail($distributor);
        }

        return $this->respondUpdated(
            new DistributorResource($distributor),
            "Distributor updated successfully!"
        );
    }

    /**
     * Remove the specified distributor from storage.
     */
    public function destroy(Distributor $distributor): JsonResponse {
        $distributor->delete();

        return $this->respondSuccess("Distributor deleted successfully!");
    }
}
