<?php

namespace App\Http\Controllers\Api\V1;

use App\Enums\ClientStatus;
use App\Enums\LeadStatus;
use App\Enums\UserRole;
use App\Http\Controllers\Controller;
use App\Http\Requests\LeadRequest;
use App\Http\Resources\LeadResource;
use App\Mail\WelcomeMail;
use App\Models\Lead;
use Illuminate\Http\JsonResponse;
use Illuminate\Support\Facades\Mail;

class LeadController extends Controller
{
    protected array $fileFieldNames = [
        "aadhar", "pan", "signature", "photo",
    ];

    protected array $fileFolderPaths = [
        "aadhar-cards", "pans", "signatures", "photos",
    ];

    protected array $relations = [
        "user", "createdBy", "assignedTo", "rm", "distributor"
    ];

    /**
     * Display a listing of the leads.
     */
    public function index(): JsonResponse {
        $leads = $this->paginateOrGet(
            Lead::with($this->relations)->filter()->latest());

        return $this->respondWithResourceCollection(
            LeadResource::collection($leads)
        );
    }

    /**
     * Store a newly created lead in storage.
     */
    public function store(LeadRequest $request): JsonResponse {
        $lead = new Lead($request->validated());
        $this->storeFiles($request, $lead);
        $lead->save();

        if ($lead->lead_status === LeadStatus::OPPORTUNITY->value) {
            $lead->update(["converted_at" => now()]);
        }

        if ($lead->client_status === ClientStatus::APPROVED->value || $lead->client_status === ClientStatus::PAYMENT_RECEIVED->value) {
            $this->createUserAndSendMail($lead);
        }

        return $this->respondCreated(
            new LeadResource($lead),
            "Lead created successfully!"
        );
    }

    /**
     * Update the specified lead in storage.
     */
    public function update(LeadRequest $request, Lead $lead): JsonResponse {
        $lead->fill($request->validated());
        $this->updateFiles($request, $lead);
        $lead->save();

        if ($lead->lead_status === LeadStatus::OPPORTUNITY->value) {
            $lead->update(["converted_at" => now()]);
        }

        if ($lead->client_status === ClientStatus::APPROVED->value && !$lead->is_mailed) {
            $this->createUserAndSendMail($lead);
        }

        return $this->respondUpdated(
            new LeadResource($lead),
            "Lead updated successfully!"
        );
    }

    private function createUserAndSendMail(Lead $lead) {
        $clientUser = $lead->user()->make($lead->toArray());
        $clientUser->password = $lead->mobile_no;
        $clientUser->role = UserRole::CLIENT->value;
        $clientUser->save();

        Mail::to($clientUser->email)->send(new WelcomeMail($clientUser));
        $lead->user_id = $clientUser->id;
        $lead->is_mailed = true;
        $lead->save();
    }

    /**
     * Display the specified lead.
     */
    public function show(Lead $lead): JsonResponse {
        $lead->loadMissing($this->relations);

        return $this->respondWithResource(new LeadResource($lead));
    }

    /**
     * Remove the specified lead from storage.
     */
    public function destroy(Lead $lead): JsonResponse {
        $lead->delete();

        return $this->respondSuccess("Lead deleted successfully!");
    }
}
